@@ -1,3 +1,17 @@
+# Copyright 2021 The google-cloud-mlflow Authors.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
 """This module provides an API for deploying MLflow models to Google Cloud Vertex AI.
 
 The `upload_mlflow_model_to_vertex_ai_models` function builds a Docker container
@@ -34,12 +48,13 @@ from typing import Any, Dict, Optional
 import urllib
 import zipfile
 
-import docker
 import google
 import google.auth
-from google.cloud.aiplatform import gapic
-from mlflow.models import cli
-from mlflow.pyfunc import scoring_server
+from google.cloud import aiplatform
+import mlflow
+from mlflow.models import cli, Model
+from mlflow.models.model import MLMODEL_FILE_NAME
+from mlflow.tracking.artifact_utils import _download_artifact_from_uri
 from unittest import mock
 
 from . import _mlflow_models_docker_utils_patch as docker_utils_patch
@@ -62,6 +77,22 @@ def get_fixed_mlflow_source_dir():
     return fixed_mlflow_dir
 
 
+def get_pickle_protocol(file_path: str) -> int:
+    import pickletools
+
+    max_proto = -1
+    with open(file_path, "rb") as file:
+        try:
+            for opcode, arg, _ in pickletools.genops(file):
+                if opcode.name == "PROTO":
+                    return arg
+                # Looking at the opcode.proto is not reliable since unsupported opcodes cannot be parsed by old python versions.
+                max_proto = max(max_proto, opcode.proto)
+        except:
+            pass
+    return max_proto
+    
+
 def upload_mlflow_model_to_vertex_ai_models(
     model_uri: str,
     display_name: str,
@@ -134,8 +165,72 @@ def upload_mlflow_model_to_vertex_ai_models(
                 " not found: {}".format(e.message)
             ) from e
 
+    temp_dir = tempfile.mkdtemp()
+    model_dir = _download_artifact_from_uri(
+        artifact_uri=model_uri, output_path=temp_dir,
+    )
+    model_metadata = Model.load(os.path.join(model_dir, MLMODEL_FILE_NAME))
+
+    for flavor_name, flavor in model_metadata.flavors.items():
+        if flavor_name == "python_function":
+            continue
+        if flavor_name == "xgboost":
+            model_file_name = flavor["data"]
+            full_xgboost_version = flavor["xgb_version"]
+            model_file_path = os.path.join(model_dir, model_file_name)
+            # TODO: Remove after https://b.corp.google.com/issues/216705259 is fixed
+            pickle_protocol = get_pickle_protocol(model_file_path)
+            # Vertex Prediction uses Python 3.7 which does not support pickle protocol 5
+            if pickle_protocol == 5:
+                _logger.warning("Detected model with pickle protocol version 5 > 4. Prebuilt containers do not support such models.")
+                continue
+            # TODO: Handle case when the version is not supported by Vertex AI
+            vertex_xgboost_version = ".".join(full_xgboost_version.split(".")[0:2])
+            vertex_model = aiplatform.Model.upload_xgboost_model_file(
+                model_file_path=model_file_path,
+                xgboost_version=vertex_xgboost_version,
+                display_name=display_name,
+                project=project,
+                location=location,
+            )
+            return vertex_model.resource_name
+        if flavor_name == "sklearn":
+            model_file_name = flavor["pickled_model"]
+            model_file_path = os.path.join(model_dir, model_file_name)
+            # TODO: Remove after https://b.corp.google.com/issues/216705259 is fixed
+            pickle_protocol = get_pickle_protocol(model_file_path)
+            # Vertex Prediction uses Python 3.7 which does not support pickle protocol 5
+            if pickle_protocol == 5:
+                _logger.warning("Detected model with pickle protocol version 5 > 4. Prebuilt containers do not support such models.")
+                continue
+            vertex_model = aiplatform.Model.upload_scikit_learn_model_file(
+                model_file_path=model_file_path,
+                # TODO: Deduce version from requirements.txt
+                # sklearn_version=
+                display_name=display_name,
+                project=project,
+                location=location,
+            )
+            return vertex_model.resource_name
+        if flavor_name == "tensorflow":
+            model_dir_name = flavor["saved_model_dir"]
+            model_dir_path = os.path.join(model_dir, model_dir_name)
+            vertex_model = aiplatform.Model.upload_tensorflow_saved_model(
+                saved_model_dir=model_dir_path,
+                # TODO: Deduce version from requirements.txt
+                # tensorflow_version=
+                display_name=display_name,
+                project=project,
+                location=location,
+            )
+            return vertex_model.resource_name
+
+    _logger.info(
+        "Model flavor is not directly supported by Vertex AI. Importing model as a custom-built container"
+    )
+
     if not destination_image_uri:
-        image_name = re.sub("[^-A-Za-z0-9_.]", "_", display_name)
+        image_name = re.sub("[^-A-Za-z0-9_.]", "_", display_name).lower()
         destination_image_uri = f"gcr.io/{project}/mlflow/{image_name}"
         _logger.info(
             "Destination image URI not set. Building and uploading image to %s",
@@ -148,14 +243,30 @@ def upload_mlflow_model_to_vertex_ai_models(
         mlflow_source_dir=None,
     )
 
-    upload_model_response = _upload_model(
-        image_uri=pushed_image_uri_with_digest,
+    uploaded_model = aiplatform.Model.upload(
+        # artifact_uri=
         display_name=display_name,
+        # description=
+        serving_container_image_uri=pushed_image_uri_with_digest,
+        # serving_container_command=
+        # serving_container_args=
+        serving_container_predict_route="/invocations",
+        serving_container_health_route="/ping",
+        # Setting environment variables to tell the scoring server to properly wrap
+        # the responses.
+        # See https://github.com/mlflow/mlflow/pull/4611
+        # https://cloud.google.com/vertex-ai/docs/predictions/custom-container-requirements#response_requirements
+        serving_container_environment_variables={
+            "PREDICTIONS_WRAPPER_ATTR_NAME": "predictions",
+        },
+        serving_container_ports=[8080],
         project=project,
         location=location,
-        model_options=model_options,
-    ).result(timeout=timeout)
-    return upload_model_response.model
+        labels={
+            "mlflow_model_vertex_ai_deployer": "mlflow_model_vertex_ai_deployer",
+        },
+    )
+    return uploaded_model.resource_name
 
 
 def _build_serving_image(
@@ -181,8 +292,8 @@ def _build_serving_image(
     )  # pylint:disable=protected-access
 
     with mock.patch(
-        "mlflow.pyfunc.backend._build_image",
-        new=docker_utils_patch._build_image
+        "mlflow.models.docker_utils._build_image_from_context",
+        new=docker_utils_patch._build_image_from_context
     ):
         flavor_backend.build_image(
             model_uri,
@@ -191,91 +302,6 @@ def _build_serving_image(
             mlflow_home=mlflow_source_dir,
         )
     return destination_image_uri
-    _logger.info("Uploading image to Google Container Registry")
-
-    client = docker.from_env()
-    result = client.images.push(destination_image_uri, stream=True, decode=True)
-    for line in result:
-        # Docker client doesn't catch auth errors, so we have to do it
-        # ourselves. See https://github.com/docker/docker-py/issues/1772
-        if "errorDetail" in line:
-            raise docker.errors.APIError(line["errorDetail"]["message"])
-        if "status" in line:
-            _logger.debug(line["status"])
-    container_image = client.images.get(destination_image_uri)
-    pushed_image_uri_with_digest = container_image.attrs["RepoDigests"][0]
-    _logger.info("Uploaded image: %s", pushed_image_uri_with_digest)
-    return pushed_image_uri_with_digest
-
-
-def _upload_model(
-    image_uri: str,
-    display_name: str,
-    model_options: Dict[str, Any],
-    project: str,
-    location: str,
-):
-    """Uploads the model with Google Cloud Vertex AI.
-
-    Args:
-      image_uri: The URI of the container image for the model.
-      display_name: The display name for the Google Cloud Vertex AI Model.
-        The name can be up to 128 characters long and can be consist of any UTF-8
-        characters.
-      model_options: A dict of other attributes of the Google Cloud Vertex AI
-        Model object (e.g. labels and schema). See
-          [Model](https://cloud.google.com/vertex-ai/docs/reference/rpc/google.cloud.aiplatform.v1#google.cloud.aiplatform.v1.Model).
-      project: The Google Cloud project where to push the container image and
-        register the model. If unset, uses the default project from gcloud.
-      location: The Google Cloud location where to push the container image and
-        register the model. Defaults to "us-central1".
-
-    Returns:
-      The full resource name of the Google Cloud Vertex AI Model.
-    """
-    # Setting environment variables to tell the scoring server to properly wrap
-    # the responses.
-    # See https://github.com/mlflow/mlflow/pull/4611
-    # https://cloud.google.com/vertex-ai/docs/predictions/custom-container-requirements#response_requirements
-    env = [
-        {
-            "name": "PREDICTIONS_WRAPPER_ATTR_NAME",
-            "value": "predictions",
-        }
-    ]
-    model_to_upload = {
-        "display_name": display_name,
-        "container_spec": {
-            "image_uri": image_uri,
-            "ports": [{"container_port": 8080}],
-            "env": env,
-            "predict_route": "/invocations",
-            "health_route": "/ping",
-        },
-    }
-    if model_options:
-        model_to_upload.update(model_options)
-    model_to_upload.setdefault("labels", {})[
-        "mlflow_model_vertex_ai_deployer"
-    ] = "mlflow_model_vertex_ai_deployer"
-
-    client_options = {
-        "api_endpoint": f"{location}-aiplatform.googleapis.com",
-    }
-
-    model_client = gapic.ModelServiceClient(client_options=client_options)
-    model_parent = f"projects/{project}/locations/{location}"
-    _logger.info(
-        "Uploading model to Google Cloud AI Platform: %s/models/%s",
-        model_parent,
-        display_name,
-    )
-    upload_model_response = model_client.upload_model(
-        parent=model_parent,
-        model=model_to_upload,
-    )
-    # model: "projects/<project_id>/locations/<location>/models/<model_id>"
-    return upload_model_response
 
 
 def deploy_vertex_ai_model_to_endpoint(
@@ -283,7 +309,7 @@ def deploy_vertex_ai_model_to_endpoint(
     endpoint_name: Optional[str] = None,
     machine_type: str = "n1-standard-2",
     min_replica_count: int = 1,
-    max_replica_count: Optional[int] = None,
+    max_replica_count: int = 1,
     endpoint_display_name: Optional[str] = None,
     deployed_model_display_name: Optional[str] = None,
     project: Optional[str] = None,
@@ -350,55 +376,29 @@ def deploy_vertex_ai_model_to_endpoint(
             model_name=model_name,
         )
     """
-    # Create an endpoint
-    # See https://github.com/googleapis/python-aiplatform/blob/master/samples/snippets/create_endpoint_sample.py
-    _, default_project = google.auth.default()
-    if not project:
-        project = default_project
-    model_id = model_name.split("/")[-1]
-
-    client_options = {
-        "api_endpoint": f"{location}-aiplatform.googleapis.com",
-    }
-    endpoint_client = gapic.EndpointServiceClient(client_options=client_options)
-    if not endpoint_name:
+    aiplatform.init(
+        project=project,
+        location=location,
+    )
+    model = aiplatform.Model(model_name)
+    if endpoint_name:
+        endpoint = aiplatform.Endpoint(endpoint_name=endpoint_name)
+    else:
+        # Model.deploy can create the Endpoint automatically, but I want to add label to the endpoint.
         if not endpoint_display_name:
-            endpoint_display_name = model_id
-        _logger.info("Creating new Endpoint: %s", endpoint_display_name)
-        endpoint_to_create = {
-            "display_name": endpoint_display_name,
-            "labels": {
+            endpoint_display_name = model.display_name[:127]
+        endpoint = aiplatform.Endpoint.create(
+            display_name=endpoint_display_name,
+            labels={
                 "mlflow_model_vertex_ai_deployer": "mlflow_model_vertex_ai_deployer",
             },
-        }
-        endpoint = endpoint_client.create_endpoint(
-            parent=f"projects/{project}/locations/{location}",
-            endpoint=endpoint_to_create,
-        ).result(timeout=timeout)
-        endpoint_name = endpoint.name
-    # projects/<prject_id>/locations/<location>/endpoints/<endpoint_id>
-    _logger.info("Endpoint name: %s", endpoint_name)
+        )
 
-    # Deploy the model
-    # See https://github.com/googleapis/python-aiplatform/blob/master/samples/snippets/deploy_model_custom_trained_model_sample.py
-    model_to_deploy = {
-        "model": model_name,
-        "display_name": deployed_model_display_name,
-        "dedicated_resources": {
-            "min_replica_count": min_replica_count,
-            "max_replica_count": max_replica_count,
-            "machine_spec": {
-                "machine_type": machine_type,
-            },
-        },
-    }
-    traffic_split = {"0": 100}
-    _logger.info(
-        "Deploying model %s to endpoint: %s", model_name, endpoint_display_name
+    return model.deploy(
+        endpoint=endpoint,
+        deployed_model_display_name=deployed_model_display_name,
+        traffic_percentage=100,
+        machine_type=machine_type,
+        min_replica_count=min_replica_count,
+        max_replica_count=max_replica_count,
     )
-    deploy_model_operation = endpoint_client.deploy_model(
-        endpoint=endpoint_name,
-        deployed_model=model_to_deploy,
-        traffic_split=traffic_split,
-    )
-    return deploy_model_operation
